<?php

namespace Mergado\Biano;

use Mergado\Tools\Languages;

class BianoStarServiceIntegration {

	/**
	 * @var BianoStarService
	 */
	private $bianoStarService;

    /**
     * @var string
     */
    private $lang;

    const BIANO_STAR_CHECKBOX = 'mmp_order_biano_star_checkbox';

	public function __construct() {
		$this->bianoStarService = new BianoStarService();
        $this->lang = Languages::getLang();
	}

    public function getService() {
        return $this->bianoStarService;
    }

    public function addCheckboxOptOut() {
        if ($this->bianoStarService->isActive($this->lang)) {
            $lang = get_locale();
            $defaultText = stripslashes($this->bianoStarService->getOptOut('en_US'));
            $checkboxText = stripslashes($this->bianoStarService->getOptOut($lang));

            if (trim($checkboxText) === '') {
                $checkboxText = $defaultText;
            }

            if (trim($checkboxText) === '') {
                $checkboxText = BianoStarService::DEFAULT_OPT;
            }

            woocommerce_form_field( self::BIANO_STAR_CHECKBOX, array( // CSS ID
                'type'          => 'checkbox',
                'class'         => array('form-row' . ' ' . self::BIANO_STAR_CHECKBOX), // CSS Class
                'label_class'   => array('woocommerce-form__label woocommerce-form__label-for-checkbox checkbox'),
                'input_class'   => array('woocommerce-form__input woocommerce-form__input-checkbox input-checkbox'),
                'required'      => false, // Mandatory or Optional
                'label'         => $checkboxText,
            ));
        }
    }

    public function setOrderMeta($orderId) {
        // Set to order meta if user want zbozi review email
        if (isset($_POST[self::BIANO_STAR_CHECKBOX]) && $_POST[self::BIANO_STAR_CHECKBOX]){
            update_post_meta( $orderId, self::BIANO_STAR_CHECKBOX, esc_attr($_POST[self::BIANO_STAR_CHECKBOX]));
        }
    }

    public function shouldBeSent($orderId) {
        if ($this->bianoStarService->isActive($this->lang)) {
            $confirmed = get_post_meta($orderId, self::BIANO_STAR_CHECKBOX, true);

            if (empty($confirmed)) {
                return true;
            }
        }

        return false;
    }
}
