<?php

class DbMigrationClass
{
    const MERGADO_DB_VERSION = 'mergado_db_version';

    /**
     * Run specific migration for site (if multisite, apply to all)
     *
     * @param string $version
     * @return void
     */
    public static function migrate(string $version) {
        if(get_option(self::MERGADO_DB_VERSION) == false || get_option(self::MERGADO_DB_VERSION) < $version) {
            include __MERGADO_UPDATES_DIR__ . sprintf('v%s.php', $version);
            self::updateDbVersion($version);
        }
    }

    /**
     * Run specific migration for site (if multisite, apply to all)
     * Do not execute migration on new instalations
     *
     *
     * @param string $version
     * @param $initalDbVersion
     * @return void
     */
    public static function migrateOnlyOnExactOrLowerVersion(string $version, $initalDbVersion) {
        if((PLUGIN_VERSION === $version && get_option(self::MERGADO_DB_VERSION) !== $version) || (get_option(self::MERGADO_DB_VERSION) !== false && get_option(self::MERGADO_DB_VERSION) < $initalDbVersion)) {
            include __MERGADO_UPDATES_DIR__ . sprintf('v%s.php', $version);
            self::updateDbVersion($version);
        }
    }

    /**
     * Run specific migration for site (if multisite, apply to all)
     *
     * @param string $version
     * @return void
     */
    public static function migrateAlwaysWithoutVersionChange(string $version) {
        include __MERGADO_UPDATES_DIR__ . sprintf('v%s.php', $version);
    }

    private static function updateDbVersion($version)
    {
        update_option(self::MERGADO_DB_VERSION, $version);
    }

    public static function getCurrentDbVersion() {
        return get_option(DbMigrationClass::MERGADO_DB_VERSION, false);
    }
}
